/*
	Public interface to TAPE device.
	Copyright 1997 Peter Urbanec
	
	Version 1.01	22 July 1997

*/

#ifndef _TAPE_H
#define _TAPE_H

#ifndef _DRIVERS_H
#include <Drivers.h>
#endif

/////////////////////////////////////////////////////////////////////
// Commands supported by the device
enum TAPE_CMD
{
	TAPE_CMD_REWIND  = B_DEVICE_OP_CODES_END + 1,
	TAPE_CMD_EJECT,
	TAPE_CMD_FILE_SEEK,
	TAPE_CMD_GET_BLOCK_SIZES,
	TAPE_CMD_GET_LAST_ERROR,
	TAPE_CMD_WRITE_FILEMARK,
	TAPE_CMD_GET_WRITE_PROTECT,
	TAPE_CMD_GET_WRITE_BUFFERING,
	TAPE_CMD_SET_WRITE_BUFFERING,
	TAPE_CMD_GET_DATA_COMPRESSION,
	TAPE_CMD_SET_DATA_COMPRESSION,
	TAPE_CMD_GET_VERSION_STRING,
	TAPE_CMD_GET_DEVICE_ID_STRING,
	TAPE_CMD_EXPERIMENTAL = B_DEVICE_OP_CODES_END + 666
};

/////////////////////////////////////////////////////////////////////
// Command structures

// Write protection setting
typedef struct
{
	bool	protection;	// true = media unwritable
} TAPE_GET_WRITE_PROTECT;

// Write buffering mode
typedef struct
{
	bool	buffering;		// true = writes are bufffered
} TAPE_WRITE_BUFFERING;

// Data compression
typedef struct
{
	bool	compression;	// true = data compressed
} TAPE_DATA_COMPRESSION;

// Seek file_count files through the tape
// Positive values go forwards, negative values go back
typedef struct
{
	int32	file_count;	// Number of files to skip
} TAPE_FILE_SEEK;

// Ask the tape drive what block sizes it supports
// A block sent to the tape must always be in the range
// returned by this command. The blocks should be even in length
// and can never be larger than 16 MB
typedef struct
{
	int32	smallest;	// Smallest block supported
	int32	largest;	// Largest block supported
} TAPE_GET_BLOCK_SIZES;

// Maximum length of an error message generated by the driver
const TAPE_MAX_MSG_LENGTH = 500;

// Returns the last error which occured while communicating with
// the tape drive. The fields are copies of the standard SCSI-2
// sense fields.
// The only fields which do not represent a SCSI-2 value are severity,
// which tries to clasify events so that the application does not need
// to care about SCSI in order to handle errors in a reasonable way and
// message which gives an ASCII string describing the problem.
typedef struct
{
	int32	severity;		// See severity definitions
	int32	residue;		// Residue length for an uncompleted command
	uint8	key;			// The class of error
	uint8	asc;			// Additional sense code
	uint8	ascq;			// Additional sense code qualifier
	char	message[TAPE_MAX_MSG_LENGTH];	// Textual error message
} TAPE_GET_LAST_ERROR;

// Available severitites
enum
{
	TAPE_SEVERITY_UNKNOWN = -1000,	// This error was not clasified,
									// the client needs to handle it

	TAPE_SEVERITY_INFO,			// Just an informational message,
								// there should have been no data loss

	TAPE_SEVERITY_WARNING,		// Something is not quite right,
								// but things can continue. There was
								// no data loss, but data might be at
								// risk if warnings persist.

	TAPE_SEVERITY_ERROR,		// Something has gone wrong. There was
								// data loss.

	TAPE_SEVERITY_FAILURE		// A key component in the system has
								// failed and the unit is out of service.
};

// Version string reporting
typedef struct
{
	// version is formated as:
	// <NAME> <VERSION>.<REVISION> (<DATE_STRING>) <COPYRIGHT>\0
	char	version[TAPE_MAX_MSG_LENGTH];
} TAPE_GET_VERSION_STRING;

// Product identification
// All fields are ASCII strings
typedef struct
{
	char	vendor[9];		// Vendor of the peripheral
	char	product[17];	// Vendor defined product string
	char	revision[5];	// Vendor defined revision string
} TAPE_GET_DEVICE_ID_STRING;

#endif  /* _TAPE_H */
